<?php

namespace NoccyLabs\TinyApp;


class ArgumentParser
{
    protected $opts;

    protected $opt_short = [];
    
    protected $opt_long = [];
    
    protected $opt_def = [];
    
    protected $opt_info = [];
    
    protected $opt_parsed = [];

    public function setOptions(array $opts)
    {
        $this->options = $opts;
    }

    private function parseOptions()
    {
        // Parse the option strings
        foreach ($this->options as $opt=>$info) {
            
            // printf("input [%s]->'%s'\n", $opt,$info);
            $short = null;
            $def = null;
            if (strpos($opt,":")!==false) {
                list($long,$def) = explode(":",$opt,2);
                $long = "{$long}:";
            } else {
                $long = $opt;
                $def = null;
            }
            // printf("  long: %s\n  def: %s\n", $long, $def);

            if (strpos($long,",")!==false) {
                list($long,$short) = explode(",",$long,2);
                if ($def) $long = "{$long}:";
            } else {
                $short = null;
            }

            // printf("  long: %s\n  short: %s\n", $long, $short);

            $this->opt_long[] = $long;
            if ($short) $this->opt_short[rtrim($short,":")] = $long;
            $this->opt_def[$long] = $def;
            $this->opt_info[$long] = $info;
            $this->opt_parsed[rtrim($long,':')] = null;
        }
        $this->opt_parsed['_params'] = [];
    }

    private function parseArgv()
    {
        $argv = array_slice($GLOBALS['argv'],1);

        // Parse the arguments
        while (count($argv)>0) {
            $arg = array_shift($argv);
            //printf("match <%s> in <%s>\n", $arg, json_encode($this->opt_parsed));
            if ($arg=="--") {
                while ($arg = array_shift($argv))
                    $this->opt_parsed['_params'][] = $arg;
            } elseif (strncmp($arg,"--",2)===0) {
                $opt = substr($arg,2);
                if (in_array("{$opt}:", $this->opt_long)) {
                    $val = array_shift($argv);
                    if ($this->opt_parsed[$opt]===null) {
                        $this->opt_parsed[$opt] = $val;
                    } else {
                        if (!is_array($this->opt_parsed[$opt])) {
                            $this->opt_parsed[$opt] = [$this->opt_parsed[$opt]];
                        }
                        $this->opt_parsed[$opt][] = $val;
                    }
                } elseif (in_array($opt, $this->opt_long)) {
                    $this->opt_parsed[$opt] = true;
                } else {
                    fprintf(STDERR, "Error: Unrecognized option: %s\n", $arg);
                    exit(1);
                }
            } elseif (strncmp($arg,"-",1)===0) {
                $opt = substr($arg,1);
                if (array_key_exists($opt,$this->opt_short)) {
                    $opt = $this->opt_short[$opt];
                    //printf("match: %s\n",$opt);
                    if (strpos($opt,":")!==false) {
                        $opt = rtrim($opt,":");
                        $val = array_shift($argv);
                    } else {
                        $val = true;
                    }
                    if (array_key_exists($opt,$this->opt_parsed) && ($this->opt_parsed[$opt] !== null)) {
                        if (!is_array($this->opt_parsed[$opt])) {
                            $this->opt_parsed[$opt] = [$this->opt_parsed[$opt]];
                        }
                        $this->opt_parsed[$opt][] = $val;
                    } else {
                        $this->opt_parsed[$opt] = $val;
                    }
                } elseif (array_key_exists($opt, $this->opt_long)) {
                    $this->opt_parsed[$opt] = true;
                } else {
                    fprintf(STDERR, "Error: Unrecognized option: %s\n", $arg);
                    exit(1);
                }
            } else {
                $this->opt_parsed['_params'][] = $arg;
            }
        }
        
    }

    public function getParsed()
    {
        if (empty($this->opt_parsed)) {
            $this->parseOptions();
            $this->parseArgv();
        }
        return $this->opt_parsed;
    }
}

